#include "enemies.h"

#include "constants.h"
#include "enemies/dragon.h"
#include "enemies/dwarf.h"
#include "enemies/elf.h"
#include "enemies/halfling.h"
#include "enemies/human.h"
#include "enemies/merchant.h"
#include "enemies/orc.h"
#include "enemies/viking.h"
#include "enemies/swordsman.h"
#include "enemies/leprechaun.h"
#include "enemies/witch.h"
#include "enemies/hacker.h"
#include "enemies/baby_dragon.h"

std::unique_ptr<enemy_base> new_dragon(RNG *rng, const position &pos,
                                       const position &fallback,
                                       const feature enabled_features,
                                       int which_room) {
        const position nil{0, 0};

        if (pos != nil)
                return std::make_unique<dragon>(rng, enabled_features,
                                                pos, which_room);
        else
                return std::make_unique<dragon>(rng, enabled_features,
                                                fallback, which_room);
}

const int EXCNT = 12;
const race EXCHOICES[EXCNT] = {
        rhuman, rdwarf, rhalfling, relf, rorc, rmerchant,
        rviking, rswordsman, rleprechaun, rwitch, rhacker, rbaby_dragon
};
const int CNT = 18;
const race CHOICES[CNT] = {
        rhuman, rhuman, rhuman, rhuman,
        rdwarf, rdwarf, rdwarf,
        rhalfling, rhalfling, rhalfling, rhalfling, rhalfling,
        relf, relf,
        rorc, rorc,
        rmerchant, rmerchant
};

enum race get_extra_race(RNG *rng) {
        return EXCHOICES[rng->rand_under(EXCNT)];
}

enum race get_normal_race(RNG *rng) {
        return CHOICES[rng->rand_under(CNT)];
}

std::unique_ptr<enemy_base> new_enemy(RNG *rng, const position &pos,
                                      const feature enabled_features,
                                      int which_room) {
        using std::make_unique;

        enum race r;

        if (enabled_features & FEATURE_EXTRA_STUFF)
                r = get_extra_race(rng);
        else
                r = get_normal_race(rng);

        switch (r) {
        case rdwarf:
                return make_unique<dwarf>(rng, enabled_features,
                                          pos, which_room);

        case rhuman:
                return make_unique<human>(rng, enabled_features,
                                          pos, which_room);

        case relf:
                return make_unique<elf>(rng, enabled_features,
                                        pos, which_room);

        case rorc:
                return make_unique<orc>(rng, enabled_features,
                                        pos, which_room);

        case rmerchant:
                return make_unique<merchant>(rng, enabled_features,
                                             pos, which_room);

        case rhalfling:
                return make_unique<halfling>(rng, enabled_features,
                                             pos, which_room);

        case rviking:
                return make_unique<viking>(rng, enabled_features,
                                           pos, which_room);

        case rswordsman:
                return make_unique<swordsman>(rng, enabled_features,
                                              pos, which_room);

        case rleprechaun:
                return make_unique<leprechaun>(rng, enabled_features,
                                               pos, which_room);

        case rwitch:
                return make_unique<witch>(rng, enabled_features,
                                          pos, which_room);

        case rhacker:
                return make_unique<hacker>(rng, enabled_features,
                                           pos, which_room);

        case rbaby_dragon:
                return make_unique<baby_dragon>(rng, enabled_features,
                                                pos, which_room);

        default:
                break;
        }

        return nullptr;
}
